class ReversiModel:
    # マスの状態を表す値
    EMPTY = 0  # 空マス
    BLACK = 1  # 黒のコマ
    WHITE = 2  # 白のコマ
    WALL = 3   # 壁

    def __init__(self):
        """
        初期化
        """
        # 全てEMPTYで盤面を作成
        self.board = [self.EMPTY] * 100

        # BLACKとWHITEのコマを配置
        self.board[44] = self.BLACK
        self.board[55] = self.BLACK
        self.board[54] = self.WHITE
        self.board[45] = self.WHITE

        # WALLを設定
        for i in range(10):
            self.board[i] = self.WALL
            self.board[90 + i] = self.WALL
            self.board[10 * i] = self.WALL
            self.board[10 * i + 9] = self.WALL
    

    def flip_list(self, i_board: int, top: int) -> list[int]:
        """
        盤面のi_boardにtop(BLACK or WHITE)を置いたときに、
        ひっくり返されるマスのリストを返す
        """
        # 空マス以外なら空のリストを返す
        if self.board[i_board] != self.EMPTY:
            return []

        # 相手のコマ
        enemy = self.BLACK if top == self.WHITE else self.WHITE

        # ひっくり返されるマスのリスト
        tops = []

        # 方向パラメータ
        UP, DOWN, LEFT, RIGHT = -10, 10, -1, 1
        
        # 8方向を走査
        for v in (RIGHT, RIGHT + UP, UP, LEFT + UP, LEFT, LEFT + DOWN, DOWN, RIGHT + DOWN):
            # ひっくり返す候補
            temp = []

            # i_boardとの差
            delta = v

            while self.board[i_board + delta] == enemy:
                # 敵のコマならひっくり返す候補に加える
                temp.append(i_board + delta)
                delta += v
            if self.board[i_board + delta] == top:
                tops += temp

        return tops


    def put(self, i_board: int, top: int) -> bool:
        """
        盤面のi_boardにtop(BLACK or WHITE)を置き、
        同じ色ではさんでいるマスをひっくり返す
        ひっくり返したらTrueを返す
        """
        tops = self.flip_list(i_board, top)
        if tops:
            self.board[i_board] = top
            for i in tops:
                self.board[i] = top
            return True

        return False
